@@ -156,8 +156,7 @@ QJsonObject &QJsonObject::operator =(const QJsonObject &other) noexcept = defaul
 /*!
     \fn void QJsonObject::swap(QJsonObject &other)
     \since 5.10
-
-    Swaps the object \a other with this. This operation is very fast and never fails.
+    \memberswap{object}
 */
 
 #ifndef QT_NO_VARIANT
@@ -1425,7 +1424,7 @@ size_t qHash(const QJsonObject &object, size_t seed)
     return seed;
 }
 
-#if !defined(QT_NO_DEBUG_STREAM) && !defined(QT_JSON_READONLY)
+#if !defined(QT_NO_DEBUG_STREAM)
 QDebug operator<<(QDebug dbg, const QJsonObject &o)
 {
     QDebugStateSaver saver(dbg);
