@@ -28,8 +28,6 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_LOGGING_CATEGORY(lcFontDb)
-
 static inline int mapToQtWeightForRange(int fcweight, int fcLower, int fcUpper, int qtLower, int qtUpper)
 {
     return qtLower + ((fcweight - fcLower) * (qtUpper - qtLower)) / (fcUpper - fcLower);
@@ -249,6 +247,13 @@ static const char specialLanguages[][6] = {
     "", // Vithkuqi
     "", // Kawi
     "", // NagMundari
+    "", // Garay
+    "", // GurungKhema
+    "", // KiratRai
+    "", // OlOnal
+    "", // Sunuwar
+    "", // Todhri
+    "", // TuluTigalari
 };
 static_assert(sizeof specialLanguages / sizeof *specialLanguages == QChar::ScriptCount);
 
@@ -477,6 +482,12 @@ static void populateFromPattern(FcPattern *pattern,
         FcPatternGetDouble (pattern, FC_PIXEL_SIZE, 0, &pixel_size);
 
     bool fixedPitch = spacing_value >= FC_MONO;
+
+    FcBool colorFont = false;
+#ifdef FC_COLOR
+    FcPatternGetBool(pattern, FC_COLOR, 0, &colorFont);
+#endif
+
     // Note: stretch should really be an int but registerFont incorrectly uses an enum
     QFont::Stretch stretch = QFont::Stretch(stretchFromFcWidth(width_value));
     QString styleName = style_value ? QString::fromUtf8((const char *) style_value) : QString();
@@ -492,7 +503,7 @@ static void populateFromPattern(FcPattern *pattern,
         applicationFont->properties.append(properties);
     }
 
-    QPlatformFontDatabase::registerFont(familyName,styleName,QLatin1StringView((const char *)foundry_value),weight,style,stretch,antialias,scalable,pixel_size,fixedPitch,writingSystems,fontFile);
+    QPlatformFontDatabase::registerFont(familyName,styleName,QLatin1StringView((const char *)foundry_value),weight,style,stretch,antialias,scalable,pixel_size,fixedPitch,colorFont,writingSystems,fontFile);
     if (applicationFont != nullptr && face != nullptr && db != nullptr) {
         db->addNamedInstancesForFace(face,
                                      indexValue,
@@ -502,6 +513,7 @@ static void populateFromPattern(FcPattern *pattern,
                                      stretch,
                                      style,
                                      fixedPitch,
+                                     colorFont,
                                      writingSystems,
                                      QByteArray((const char*)file_value),
                                      applicationFont->data);
@@ -538,7 +550,7 @@ static void populateFromPattern(FcPattern *pattern,
                 applicationFont->properties.append(properties);
             }
             FontFile *altFontFile = new FontFile(*fontFile);
-            QPlatformFontDatabase::registerFont(altFamilyName, altStyleName, QLatin1StringView((const char *)foundry_value),weight,style,stretch,antialias,scalable,pixel_size,fixedPitch,writingSystems,altFontFile);
+            QPlatformFontDatabase::registerFont(altFamilyName, altStyleName, QLatin1StringView((const char *)foundry_value),weight,style,stretch,antialias,scalable,pixel_size,fixedPitch,colorFont,writingSystems,altFontFile);
         } else {
             QPlatformFontDatabase::registerAliasToFontFamily(familyName, altFamilyName);
         }
@@ -548,7 +560,7 @@ static void populateFromPattern(FcPattern *pattern,
 
 static bool isDprScaling()
 {
-    return !qFuzzyCompare(qApp->devicePixelRatio(), 1.0);
+    return !qFuzzyCompare(qApp->devicePixelRatio(), qreal(1.0));
 }
 
 QFontconfigDatabase::~QFontconfigDatabase()
@@ -571,6 +583,9 @@ void QFontconfigDatabase::populateFontDatabase()
             FC_WIDTH, FC_FAMILYLANG,
 #if FC_VERSION >= 20297
             FC_CAPABILITY,
+#endif
+#if defined(FC_COLOR)
+            FC_COLOR,
 #endif
             (const char *)nullptr
         };
@@ -615,9 +630,9 @@ void QFontconfigDatabase::populateFontDatabase()
 
     while (f->qtname) {
         QString familyQtName = QString::fromLatin1(f->qtname);
-        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleNormal,QFont::Unstretched,true,true,0,f->fixed,ws,nullptr);
-        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleItalic,QFont::Unstretched,true,true,0,f->fixed,ws,nullptr);
-        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleOblique,QFont::Unstretched,true,true,0,f->fixed,ws,nullptr);
+        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleNormal,QFont::Unstretched,true,true,0,f->fixed,false,ws,nullptr);
+        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleItalic,QFont::Unstretched,true,true,0,f->fixed,false,ws,nullptr);
+        registerFont(familyQtName,QString(),QString(),QFont::Normal,QFont::StyleOblique,QFont::Unstretched,true,true,0,f->fixed,false,ws,nullptr);
         ++f;
     }
 
@@ -636,13 +651,13 @@ void QFontconfigDatabase::invalidate()
     FcConfigAppFontClear(nullptr);
 }
 
-QFontEngineMulti *QFontconfigDatabase::fontEngineMulti(QFontEngine *fontEngine, QChar::Script script)
+QFontEngineMulti *QFontconfigDatabase::fontEngineMulti(QFontEngine *fontEngine, QFontDatabasePrivate::ExtendedScript script)
 {
     return new QFontEngineMultiFontConfig(fontEngine, script);
 }
 
 namespace {
-QFontEngine::HintStyle defaultHintStyleFromMatch(QFont::HintingPreference hintingPreference, FcPattern *match, bool useXftConf)
+QFontEngine::HintStyle defaultHintStyleFromMatch(QFont::HintingPreference hintingPreference, FcPattern *match, bool preferXftConf)
 {
     switch (hintingPreference) {
     case QFont::PreferNoHinting:
@@ -658,6 +673,13 @@ QFontEngine::HintStyle defaultHintStyleFromMatch(QFont::HintingPreference hintin
     if (isDprScaling())
         return QFontEngine::HintNone;
 
+    void *hintStyleResource =
+            QGuiApplication::platformNativeInterface()->nativeResourceForScreen("hintstyle",
+                                                                                QGuiApplication::primaryScreen());
+    int xftHintStyle =  int(reinterpret_cast<qintptr>(hintStyleResource));
+    if (preferXftConf && xftHintStyle > 0)
+        return QFontEngine::HintStyle(xftHintStyle - 1);
+
     int hint_style = 0;
     if (FcPatternGetInteger (match, FC_HINT_STYLE, 0, &hint_style) == FcResultMatch) {
         switch (hint_style) {
@@ -674,21 +696,21 @@ QFontEngine::HintStyle defaultHintStyleFromMatch(QFont::HintingPreference hintin
             break;
         }
     }
-
-    if (useXftConf) {
-        void *hintStyleResource =
-                QGuiApplication::platformNativeInterface()->nativeResourceForScreen("hintstyle",
-                                                                                    QGuiApplication::primaryScreen());
-        int hintStyle = int(reinterpret_cast<qintptr>(hintStyleResource));
-        if (hintStyle > 0)
-            return QFontEngine::HintStyle(hintStyle - 1);
-    }
+    if (xftHintStyle > 0)
+        return QFontEngine::HintStyle(xftHintStyle - 1);
 
     return QFontEngine::HintFull;
 }
 
-QFontEngine::SubpixelAntialiasingType subpixelTypeFromMatch(FcPattern *match, bool useXftConf)
+QFontEngine::SubpixelAntialiasingType subpixelTypeFromMatch(FcPattern *match, bool preferXftConf)
 {
+    void *subpixelTypeResource =
+            QGuiApplication::platformNativeInterface()->nativeResourceForScreen("subpixeltype",
+                                                                                QGuiApplication::primaryScreen());
+    int xftSubpixelType = int(reinterpret_cast<qintptr>(subpixelTypeResource));
+    if (preferXftConf && xftSubpixelType > 0)
+        return QFontEngine::SubpixelAntialiasingType(xftSubpixelType - 1);
+
     int subpixel = FC_RGBA_UNKNOWN;
     if (FcPatternGetInteger(match, FC_RGBA, 0, &subpixel) == FcResultMatch) {
         switch (subpixel) {
@@ -709,14 +731,8 @@ QFontEngine::SubpixelAntialiasingType subpixelTypeFromMatch(FcPattern *match, bo
         }
     }
 
-    if (useXftConf) {
-        void *subpixelTypeResource =
-                QGuiApplication::platformNativeInterface()->nativeResourceForScreen("subpixeltype",
-                                                                                    QGuiApplication::primaryScreen());
-        int subpixelType = int(reinterpret_cast<qintptr>(subpixelTypeResource));
-        if (subpixelType > 0)
-            return QFontEngine::SubpixelAntialiasingType(subpixelType - 1);
-    }
+    if (xftSubpixelType > 0)
+        return QFontEngine::SubpixelAntialiasingType(xftSubpixelType - 1);
 
     return QFontEngine::Subpixel_None;
 }
@@ -759,7 +775,10 @@ QFontEngine *QFontconfigDatabase::fontEngine(const QByteArray &fontData, qreal p
     return engine;
 }
 
-QStringList QFontconfigDatabase::fallbacksForFamily(const QString &family, QFont::Style style, QFont::StyleHint styleHint, QChar::Script script) const
+QStringList QFontconfigDatabase::fallbacksForFamily(const QString &family,
+                                                    QFont::Style style,
+                                                    QFont::StyleHint styleHint,
+                                                    QFontDatabasePrivate::ExtendedScript script) const
 {
     QStringList fallbackFamilies;
     FcPattern *pattern = FcPatternCreate();
@@ -772,6 +791,14 @@ QStringList QFontconfigDatabase::fallbacksForFamily(const QString &family, QFont
     value.u.s = (const FcChar8 *)cs.data();
     FcPatternAdd(pattern,FC_FAMILY,value,true);
 
+#ifdef FC_COLOR
+    if (script == QFontDatabasePrivate::Script_Emoji) {
+        FcPatternAddBool(pattern, FC_COLOR, true);
+        value.u.s = (const FcChar8 *)"emoji";
+        FcPatternAddWeak(pattern, FC_FAMILY, value, FcTrue);
+    }
+#endif
+
     int slant_value = FC_SLANT_ROMAN;
     if (style == QFont::StyleItalic)
         slant_value = FC_SLANT_ITALIC;
@@ -779,8 +806,8 @@ QStringList QFontconfigDatabase::fallbacksForFamily(const QString &family, QFont
         slant_value = FC_SLANT_OBLIQUE;
     FcPatternAddInteger(pattern, FC_SLANT, slant_value);
 
-    Q_ASSERT(uint(script) < QChar::ScriptCount);
-    if (*specialLanguages[script] != '\0') {
+    Q_ASSERT(uint(script) < QFontDatabasePrivate::ScriptCount);
+    if (uint(script) < QChar::ScriptCount && *specialLanguages[script] != '\0') {
         FcLangSet *ls = FcLangSetCreate();
         FcLangSetAdd(ls, (const FcChar8*)specialLanguages[script]);
         FcPatternAddLangSet(pattern, FC_LANG, ls);
@@ -965,20 +992,11 @@ void QFontconfigDatabase::setupFontEngine(QFontEngineFT *engine, const QFontDef
     bool forcedAntialiasSetting = !antialias || isDprScaling();
 
     const QPlatformServices *services = QGuiApplicationPrivate::platformIntegration()->services();
-    bool useXftConf = false;
+    bool preferXftConf = false;
 
     if (services) {
         const QList<QByteArray> desktopEnv = services->desktopEnvironment().split(':');
-        useXftConf = desktopEnv.contains("GNOME") || desktopEnv.contains("UNITY") || desktopEnv.contains("XFCE");
-    }
-
-    if (useXftConf && !forcedAntialiasSetting) {
-        void *antialiasResource =
-                QGuiApplication::platformNativeInterface()->nativeResourceForScreen("antialiasingEnabled",
-                                                                                    QGuiApplication::primaryScreen());
-        int antialiasingEnabled = int(reinterpret_cast<qintptr>(antialiasResource));
-        if (antialiasingEnabled > 0)
-            antialias = antialiasingEnabled - 1;
+        preferXftConf = !(desktopEnv.contains("KDE") || desktopEnv.contains("LXQT") || desktopEnv.contains("UKUI"));
     }
 
     QFontEngine::GlyphFormat format;
@@ -1060,8 +1078,19 @@ bail:
     if (!match)
         match = FcFontMatch(nullptr, pattern, &result);
 
+    int xftAntialias = 0;
+    if (!forcedAntialiasSetting) {
+        void *antialiasResource =
+                QGuiApplication::platformNativeInterface()->nativeResourceForScreen("antialiasingEnabled",
+                                                                                    QGuiApplication::primaryScreen());
+        xftAntialias = int(reinterpret_cast<qintptr>(antialiasResource));
+        if ((preferXftConf || !match) && xftAntialias > 0) {
+            antialias = xftAntialias - 1;
+            forcedAntialiasSetting = true;
+        }
+    }
     if (match) {
-        engine->setDefaultHintStyle(defaultHintStyleFromMatch((QFont::HintingPreference)fontDef.hintingPreference, match, useXftConf));
+        engine->setDefaultHintStyle(defaultHintStyleFromMatch((QFont::HintingPreference)fontDef.hintingPreference, match, preferXftConf));
 
         FcBool fc_autohint;
         if (FcPatternGetBool(match, FC_AUTOHINT,0, &fc_autohint) == FcResultMatch)
@@ -1082,18 +1111,37 @@ bail:
         if (antialias) {
             QFontEngine::SubpixelAntialiasingType subpixelType = QFontEngine::Subpixel_None;
             if (!(fontDef.styleStrategy & QFont::NoSubpixelAntialias))
-                subpixelType = subpixelTypeFromMatch(match, useXftConf);
+                subpixelType = subpixelTypeFromMatch(match, preferXftConf);
             engine->subpixelType = subpixelType;
-
-            format = (subpixelType == QFontEngine::Subpixel_None)
-                    ? QFontEngine::Format_A8
-                    : QFontEngine::Format_A32;
-        } else
-            format = QFontEngine::Format_Mono;
+        }
 
         FcPatternDestroy(match);
-    } else
-        format = antialias ? QFontEngine::Format_A8 : QFontEngine::Format_Mono;
+    } else {
+        void *hintStyleResource =
+                QGuiApplication::platformNativeInterface()->nativeResourceForScreen("hintstyle",
+                                                                                    QGuiApplication::primaryScreen());
+        int xftHintStyle =  int(reinterpret_cast<qintptr>(hintStyleResource));
+        if (xftHintStyle > 0)
+            engine->setDefaultHintStyle(QFontEngine::HintStyle(xftHintStyle - 1));
+        if (antialias) {
+            engine->subpixelType = QFontEngine::Subpixel_None;
+            if (!(fontDef.styleStrategy & QFont::NoSubpixelAntialias)) {
+                void *subpixelTypeResource =
+                        QGuiApplication::platformNativeInterface()->nativeResourceForScreen("subpixeltype",
+                                                                                            QGuiApplication::primaryScreen());
+                int xftSubpixelType = int(reinterpret_cast<qintptr>(subpixelTypeResource));
+                if (xftSubpixelType > 1)
+                    engine->subpixelType = QFontEngine::SubpixelAntialiasingType(xftSubpixelType - 1);
+            }
+        }
+    }
+    if (antialias) {
+        format = (engine->subpixelType == QFontEngine::Subpixel_None)
+                ? QFontEngine::Format_A8
+                : QFontEngine::Format_A32;
+    } else {
+        format = QFontEngine::Format_Mono;
+    }
 
     FcPatternDestroy(pattern);
 
