@@ -13,7 +13,7 @@
 #include <QOperatingSystemVersion>
 #endif
 
-#if defined(Q_OS_WIN) && !QT_CONFIG(icu)
+#if defined(Q_OS_WIN) && !QT_CONFIG(icu) && !QT_CONFIG(timezone_tzdb)
 #  define USING_WIN_TZ
 #endif
 
@@ -70,11 +70,32 @@ private Q_SLOTS:
 
 private:
     void printTimeZone(const QTimeZone &tz);
-#if defined(QT_BUILD_INTERNAL) && QT_CONFIG(timezone)
+#if QT_CONFIG(timezone)
+#  if defined(QT_BUILD_INTERNAL)
     // Generic tests of privates, called by implementation-specific private tests:
     void testCetPrivate(const QTimeZonePrivate &tzp);
     void testEpochTranPrivate(const QTimeZonePrivate &tzp);
-#endif // QT_BUILD_INTERNAL && timezone backends
+#  endif // QT_BUILD_INTERNAL
+    // Where tzdb contains a link between zones in different territories, CLDR
+    // doesn't treat those as aliases for one another. For details see "Links in
+    // the tz database" at:
+    // https://www.unicode.org/reports/tr35/#time-zone-identifiers
+    // Some of these could be identified as equivalent by looking at metazone
+    // histories but, for now, we stick with CLDR's notion of alias.
+    const std::set<QByteArrayView> unAliasedLinks = {
+        // By continent:
+        "America/Kralendijk", "America/Lower_Princes", "America/Marigot", "America/St_Barthelemy",
+        "Antarctica/South_Pole",
+        "Arctic/Longyearbyen",
+        "Asia/Choibalsan",
+        "Atlantic/Jan_Mayen",
+        "Europe/Bratislava", "Europe/Busingen", "Europe/Mariehamn",
+        "Europe/Podgorica", "Europe/San_Marino", "Europe/Vatican",
+        // Assorted legacy abbreviations and POSIX zones:
+        "CET", "EET", "EST", "HST", "MET", "MST", "WET",
+        "CST6CDT", "EST5EDT", "MST7MDT", "PST8PDT",
+    };
+#endif // timezone backends
     // Set to true to print debug output, test Display Names and run long stress tests
     static constexpr bool debug = false;
 };
@@ -149,7 +170,7 @@ void tst_QTimeZone::createTest()
 {
     const QTimeZone tz("Pacific/Auckland");
 
-    if (debug)
+    if constexpr (debug)
         printTimeZone(tz);
 
     // If the tz is not valid then skip as is probably using the UTC backend which is tested later
@@ -565,7 +586,8 @@ void tst_QTimeZone::isTimeZoneIdAvailable()
         QVERIFY2(QTimeZone::isTimeZoneIdAvailable(id), id);
         const QTimeZone zone(id);
         QVERIFY2(zone.isValid(), id);
-        QVERIFY2(zone.hasAlternativeName(id), zone.id() + " != " + id);
+        if (unAliasedLinks.find(id) == unAliasedLinks.end())
+            QVERIFY2(zone.hasAlternativeName(id), zone.id() + " != " + id);
     }
     // availableTimeZoneIds() doesn't list all possible offset IDs, but
     // isTimeZoneIdAvailable() should accept them.
@@ -762,6 +784,13 @@ void tst_QTimeZone::hasAlternativeName()
 
 void tst_QTimeZone::specificTransition_data()
 {
+#if QT_CONFIG(timezone) && QT_CONFIG(timezone_tzdb) && defined(__GLIBCXX__)
+    QSKIP("libstdc++'s C++20 misreads the IANA DB for Moscow's transitions (among others).");
+#endif
+#if defined Q_OS_ANDROID && !QT_CONFIG(timezone_tzdb)
+    if (!QTimeZone("Europe/Moscow").hasTransitions())
+        QSKIP("Android time-zone back-end has no transition data");
+#endif
     QTest::addColumn<QByteArray>("zone");
     QTest::addColumn<QDate>("start");
     QTest::addColumn<QDate>("stop");
@@ -771,10 +800,6 @@ void tst_QTimeZone::specificTransition_data()
     QTest::addColumn<int>("offset");
     QTest::addColumn<int>("stdoff");
     QTest::addColumn<int>("dstoff");
-#ifdef Q_OS_ANDROID
-    if (!QTimeZone("Europe/Moscow").hasTransitions())
-        QSKIP("Android time-zone back-end has no transition data");
-#endif
 
     // Moscow ditched DST on 2010-10-31 but has since changed standard offset twice.
 #ifdef USING_WIN_TZ
@@ -958,7 +983,7 @@ void tst_QTimeZone::checkOffset()
 
 void tst_QTimeZone::availableTimeZoneIds()
 {
-    if (debug) {
+    if constexpr (debug) {
         qDebug() << "";
         qDebug() << "Available Time Zones" ;
         qDebug() << QTimeZone::availableTimeZoneIds();
@@ -970,10 +995,21 @@ void tst_QTimeZone::availableTimeZoneIds()
         qDebug() << QTimeZone::availableTimeZoneIds(0);
         qDebug() << "";
     } else {
-        //Just test the calls work, we cannot know what any test machine has available
+        // Test the calls work:
         QList<QByteArray> listAll = QTimeZone::availableTimeZoneIds();
-        QList<QByteArray> listUs = QTimeZone::availableTimeZoneIds(QLocale::UnitedStates);
-        QList<QByteArray> listZero = QTimeZone::availableTimeZoneIds(0);
+        QList<QByteArray> list001 = QTimeZone::availableTimeZoneIds(QLocale::World);
+        QList<QByteArray> listUsa = QTimeZone::availableTimeZoneIds(QLocale::UnitedStates);
+        QList<QByteArray> listGmt = QTimeZone::availableTimeZoneIds(0);
+        // We cannot know what any test machine has available, so can't test contents.
+        // But we can do a consistency check:
+        QCOMPARE_LT(list001.size(), listAll.size());
+        QCOMPARE_LT(listUsa.size(), listAll.size());
+        QCOMPARE_LT(listGmt.size(), listAll.size());
+        // And we do know CLDR data supplies some entries to each:
+        QCOMPARE_GT(listAll.size(), 0);
+        QCOMPARE_GT(list001.size(), 0);
+        QCOMPARE_GT(listUsa.size(), 0);
+        QCOMPARE_GT(listGmt.size(), 0);
     }
 }
 
@@ -984,7 +1020,8 @@ void tst_QTimeZone::stressTest()
     for (const QByteArray &id : idList) {
         QTimeZone testZone = QTimeZone(id);
         QCOMPARE(testZone.isValid(), true);
-        QVERIFY2(testZone.hasAlternativeName(id), testZone.id() + " != " + id);
+        if (unAliasedLinks.find(id) == unAliasedLinks.end())
+            QVERIFY2(testZone.hasAlternativeName(id), testZone.id() + " != " + id);
         QDateTime testDate = QDateTime(QDate(2015, 1, 1), QTime(0, 0), UTC);
         testZone.territory();
         testZone.comment();
@@ -1014,7 +1051,7 @@ void tst_QTimeZone::stressTest()
         testZone.nextTransition(highDate2);
         testZone.previousTransition(highDate1);
         testZone.previousTransition(highDate2);
-        if (debug) {
+        if constexpr (debug) {
             // This could take a long time, depending on platform and database
             qDebug() << "Stress test calculating transistions for" << testZone.id();
             testZone.transitions(lowDate1, highDate1);
@@ -1031,13 +1068,12 @@ void tst_QTimeZone::windowsId()
 /*
     Current Windows zones for "Central Standard Time":
     Region      IANA Id(s)
-    Default     "America/Chicago"
+    World       "America/Chicago" (the default)
     Canada      "America/Winnipeg America/Rankin_Inlet America/Resolute"
-    Mexico      "America/Matamoros"
+    Mexico      "America/Matamoros America/Ojinaga"
     USA         "America/Chicago America/Indiana/Knox America/Indiana/Tell_City America/Menominee"
                 "America/North_Dakota/Beulah America/North_Dakota/Center"
                 "America/North_Dakota/New_Salem"
-    AnyTerritory  "CST6CDT"
 */
     QCOMPARE(QTimeZone::ianaIdToWindowsId("America/Chicago"),
              QByteArray("Central Standard Time"));
@@ -1052,10 +1088,12 @@ void tst_QTimeZone::windowsId()
     // Check default value
     QCOMPARE(QTimeZone::windowsIdToDefaultIanaId("Central Standard Time"),
              QByteArray("America/Chicago"));
+    QCOMPARE(QTimeZone::windowsIdToDefaultIanaId("Central Standard Time", QLocale::World),
+             QByteArray("America/Chicago"));
     QCOMPARE(QTimeZone::windowsIdToDefaultIanaId("Central Standard Time", QLocale::Canada),
              QByteArray("America/Winnipeg"));
     QCOMPARE(QTimeZone::windowsIdToDefaultIanaId("Central Standard Time", QLocale::AnyTerritory),
-             QByteArray("CST6CDT"));
+             QByteArray());
     QCOMPARE(QTimeZone::windowsIdToDefaultIanaId(QByteArray()), QByteArray());
 
     {
@@ -1065,10 +1103,15 @@ void tst_QTimeZone::windowsId()
             "America/Matamoros", "America/Menominee", "America/North_Dakota/Beulah",
             "America/North_Dakota/Center", "America/North_Dakota/New_Salem",
             "America/Ojinaga", "America/Rankin_Inlet", "America/Resolute",
-            "America/Winnipeg", "CST6CDT"
+            "America/Winnipeg"
         };
         QCOMPARE(QTimeZone::windowsIdToIanaIds("Central Standard Time"), list);
     }
+    {
+        const QList<QByteArray> list = { "America/Chicago" };
+        QCOMPARE(QTimeZone::windowsIdToIanaIds("Central Standard Time", QLocale::World),
+                 list);
+    }
     {
         // Check country with no match returns empty list
         const QList<QByteArray> empty;
@@ -1096,7 +1139,7 @@ void tst_QTimeZone::windowsId()
                  list);
     }
     {
-        const QList<QByteArray> list = { "CST6CDT" };
+        const QList<QByteArray> list;
         QCOMPARE(QTimeZone::windowsIdToIanaIds("Central Standard Time", QLocale::AnyTerritory),
                  list);
     }
@@ -1128,7 +1171,7 @@ void tst_QTimeZone::isValidId_data()
     // Parts separated by '/', each part min 1 and max of 14 chars
     TESTSET("empty", "", false);
     TESTSET("minimal", "m", true);
-#if defined(Q_OS_ANDROID) || QT_CONFIG(icu)
+#if (defined(Q_OS_ANDROID) || QT_CONFIG(icu)) && !QT_CONFIG(timezone_tzdb)
     TESTSET("maximal", "East-Saskatchewan", true); // Android actually uses this
     TESTSET("too long", "North-Saskatchewan", false); // ... but thankfully not this.
 #else
@@ -1199,7 +1242,7 @@ void tst_QTimeZone::isValidId_data()
     QTest::newRow("a,z alone") << QByteArray("a,z") << false;
     QTest::newRow("/z alone") << QByteArray("/z") << false;
     QTest::newRow("-z alone") << QByteArray("-z") << false;
-#if defined(Q_OS_ANDROID) || QT_CONFIG(icu)
+#if (defined(Q_OS_ANDROID) || QT_CONFIG(icu)) && !QT_CONFIG(timezone_tzdb)
     QTest::newRow("long alone") << QByteArray("12345678901234567") << true;
     QTest::newRow("over-long alone") << QByteArray("123456789012345678") << false;
 #else
@@ -1340,7 +1383,7 @@ void tst_QTimeZone::utcTest()
 
 void tst_QTimeZone::icuTest()
 {
-#if defined(QT_BUILD_INTERNAL) && QT_CONFIG(icu) && !defined(Q_OS_UNIX)
+#if defined(QT_BUILD_INTERNAL) && QT_CONFIG(icu) && !QT_CONFIG(timezone_tzdb) && !defined(Q_OS_UNIX)
     // Known datetimes
     qint64 std = QDateTime(QDate(2012, 1, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
     qint64 dst = QDateTime(QDate(2012, 6, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
@@ -1360,7 +1403,7 @@ void tst_QTimeZone::icuTest()
     QVERIFY(tzp.isValid());
 
     // Only test names in debug mode, names used can vary by ICU version installed
-    if (debug) {
+    if constexpr (debug) {
         // Test display names by type
         QLocale enUS("en_US");
         ZONE_DNAME_CHECK(StandardTime, LongName, "Central European Standard Time");
@@ -1383,12 +1426,13 @@ void tst_QTimeZone::icuTest()
     if (QTest::currentTestFailed())
         return;
     testEpochTranPrivate(QIcuTimeZonePrivate("America/Toronto"));
-#endif // ICU not on Unix
+#endif // ICU not on Unix, without tzdb
 }
 
 void tst_QTimeZone::tzTest()
 {
-#if defined QT_BUILD_INTERNAL && defined Q_OS_UNIX && !defined Q_OS_DARWIN && !defined Q_OS_ANDROID
+#if defined QT_BUILD_INTERNAL && defined Q_OS_UNIX \
+        && !QT_CONFIG(timezone_tzdb) && !defined Q_OS_DARWIN && !defined Q_OS_ANDROID
     const auto UTC = QTimeZone::UTC;
     // Known datetimes
     qint64 std = QDateTime(QDate(2012, 1, 1), QTime(0, 0), UTC).toMSecsSinceEpoch();
@@ -1436,7 +1480,7 @@ void tst_QTimeZone::tzTest()
     // Test display names by type, either ICU or abbreviation only
     QLocale enUS("en_US");
     // Only test names in debug mode, names used can vary by ICU version installed
-    if (debug) {
+    if constexpr (debug) {
 #if QT_CONFIG(icu)
         ZONE_DNAME_CHECK(StandardTime, LongName, "Central European Standard Time");
         ZONE_DNAME_CHECK(StandardTime, ShortName, "GMT+01:00");
@@ -1587,12 +1631,12 @@ void tst_QTimeZone::tzTest()
         QDateTime dt(QDate(2016, 3, 28), QTime(0, 0), UTC);
         QCOMPARE(tzBarnaul.data(dt.toMSecsSinceEpoch()).abbreviation, QString("+07"));
     }
-#endif // QT_BUILD_INTERNAL && Q_OS_UNIX && !Q_OS_DARWIN && !Q_OS_ANDROID
+#endif // QT_BUILD_INTERNAL && Q_OS_UNIX && !timezone_tzdb && !Q_OS_DARWIN && !Q_OS_ANDROID
 }
 
 void tst_QTimeZone::macTest()
 {
-#if defined(QT_BUILD_INTERNAL) && defined(Q_OS_DARWIN)
+#if defined(QT_BUILD_INTERNAL) && defined(Q_OS_DARWIN) && !QT_CONFIG(timezone_tzdb)
     // Known datetimes
     qint64 std = QDateTime(QDate(2012, 1, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
     qint64 dst = QDateTime(QDate(2012, 6, 1), QTime(0, 0), QTimeZone::UTC).toMSecsSinceEpoch();
@@ -1610,7 +1654,7 @@ void tst_QTimeZone::macTest()
     QVERIFY(tzp.isValid());
 
     // Only test names in debug mode, names used can vary by version
-    if (debug) {
+    if constexpr (debug) {
         // Test display names by type
         QLocale enUS("en_US");
         ZONE_DNAME_CHECK(StandardTime, LongName, "Central European Standard Time");
@@ -1633,7 +1677,7 @@ void tst_QTimeZone::macTest()
     if (QTest::currentTestFailed())
         return;
     testEpochTranPrivate(QMacTimeZonePrivate("America/Toronto"));
-#endif // QT_BUILD_INTERNAL && Q_OS_DARWIN
+#endif // QT_BUILD_INTERNAL && Q_OS_DARWIN without tzdb
 }
 
 void tst_QTimeZone::darwinTypes()
@@ -1655,7 +1699,7 @@ void tst_QTimeZone::winTest()
 
     // Test default constructor
     QWinTimeZonePrivate tzpd;
-    if (debug)
+    if constexpr (debug)
         qDebug() << "System ID = " << tzpd.id()
                  << tzpd.displayName(QTimeZone::StandardTime, QTimeZone::LongName, QLocale())
                  << tzpd.displayName(QTimeZone::GenericTime, QTimeZone::LongName, QLocale());
@@ -1670,7 +1714,7 @@ void tst_QTimeZone::winTest()
     QVERIFY(tzp.isValid());
 
     // Only test names in debug mode, names used can vary by version
-    if (debug) {
+    if constexpr (debug) {
         // Test display names by type
         QLocale enUS("en_US");
         ZONE_DNAME_CHECK(StandardTime, LongName, "W. Europe Standard Time");
@@ -1686,8 +1730,8 @@ void tst_QTimeZone::winTest()
         ZONE_DNAME_CHECK(GenericTime, OffsetName, "UTC+01:00");
 
         // Test Abbreviations
-        QCOMPARE(tzp.abbreviation(std), QString("W. Europe Standard Time"));
-        QCOMPARE(tzp.abbreviation(dst), QString("W. Europe Daylight Time"));
+        QCOMPARE(tzp.abbreviation(std), QString("CET"));
+        QCOMPARE(tzp.abbreviation(dst), QString("CEST"));
     }
 
     testCetPrivate(tzp);
@@ -1701,9 +1745,6 @@ void tst_QTimeZone::winTest()
 
 void tst_QTimeZone::localeSpecificDisplayName_data()
 {
-#ifdef USING_WIN_TZ
-    QSKIP("MS backend does not use locale parameter");
-#endif
     QTest::addColumn<QByteArray>("zoneName");
     QTest::addColumn<QLocale>("locale");
     QTest::addColumn<QTimeZone::TimeType>("timeType");
@@ -1745,8 +1786,6 @@ void tst_QTimeZone::localeSpecificDisplayName()
     QVERIFY(zone.isValid());
 
     const QString localeName = zone.displayName(timeType, QTimeZone::LongName, locale);
-    if (localeName.isEmpty()) // Backend doesn't know how to localize this zone's name
-        QEXPECT_FAIL("", "QTBUG-115158 zone name localization unknown", Continue);
     QCOMPARE(localeName, expectedName);
 }
 
