@@ -48,8 +48,6 @@
 
 #include <algorithm>
 
-Q_LOGGING_CATEGORY(lcTests, "qt.widgets.tests")
-
 QT_BEGIN_NAMESPACE
 
 extern bool Q_GUI_EXPORT qt_tab_all_widgets(); // from qapplication.cpp
@@ -59,6 +57,7 @@ class tst_QApplication : public QObject
 {
 Q_OBJECT
 
+    void runHelperTest();
 private slots:
     void cleanup();
     void sendEventsOnProcessEvents(); // this must be the first test
@@ -89,16 +88,17 @@ private slots:
     void testDeleteLaterProcessEvents4();
     void testDeleteLaterProcessEvents5();
 
-#if QT_CONFIG(library)
-    void libraryPaths();
-    void libraryPaths_qt_plugin_path();
-    void libraryPaths_qt_plugin_path_2();
-#endif
-
 #ifdef QT_BUILD_INTERNAL
     void sendPostedEvents();
 #endif  // ifdef QT_BUILD_INTERNAL
 
+#if QT_CONFIG(process)
+    void exitFromEventLoop() { runHelperTest(); }
+    void exitFromThread() { runHelperTest(); }
+    void exitFromThreadedEventLoop() { runHelperTest(); }
+    void exitWithPlugins() { runHelperTest(); }
+    void mainAppInAThread() { runHelperTest(); }
+#endif
     void thread();
     void desktopSettingsAware();
 
@@ -946,203 +946,6 @@ void tst_QApplication::closeAllWindows()
     qDeleteAll(QApplication::topLevelWidgets());
 }
 
-bool isPathListIncluded(const QStringList &l, const QStringList &r)
-{
-    int size = r.size();
-    if (size > l.size())
-        return false;
-#if defined (Q_OS_WIN)
-    Qt::CaseSensitivity cs = Qt::CaseInsensitive;
-#else
-    Qt::CaseSensitivity cs = Qt::CaseSensitive;
-#endif
-    int i = 0, j = 0;
-    for ( ; i < l.size() && j < r.size(); ++i) {
-        if (QDir::toNativeSeparators(l[i]).compare(QDir::toNativeSeparators(r[j]), cs) == 0) {
-            ++j;
-            i = -1;
-        }
-    }
-    return j == r.size();
-}
-
-#if QT_CONFIG(library)
-void tst_QApplication::libraryPaths()
-{
-#ifndef BUILTIN_TESTDATA
-        const QString testDir = QFileInfo(QFINDTESTDATA("test/CMakeLists.txt")).absolutePath();
-#else
-        const QString testDir = QFileInfo(QFINDTESTDATA("CMakeLists.txt")).absolutePath();
-#endif
-        QVERIFY(!testDir.isEmpty());
-    {
-        QApplication::setLibraryPaths(QStringList() << testDir);
-        QCOMPARE(QApplication::libraryPaths(), (QStringList() << testDir));
-
-        // creating QApplication adds the applicationDirPath to the libraryPath
-        int argc = 1;
-        QApplication app(argc, &argv0);
-        QString appDirPath = QDir(QCoreApplication::applicationDirPath()).canonicalPath();
-
-        QStringList actual = QApplication::libraryPaths();
-        actual.sort();
-        QStringList expected;
-        expected << testDir << appDirPath;
-        expected = QSet<QString>(expected.constBegin(), expected.constEnd()).values();
-        expected.sort();
-
-        QVERIFY2(isPathListIncluded(actual, expected),
-                 qPrintable("actual:\n - " + actual.join("\n - ") +
-                            "\nexpected:\n - " + expected.join("\n - ")));
-    }
-    {
-        // creating QApplication adds the applicationDirPath and plugin install path to the libraryPath
-        int argc = 1;
-        QApplication app(argc, &argv0);
-        QString appDirPath = QCoreApplication::applicationDirPath();
-        QString installPathPlugins =  QLibraryInfo::path(QLibraryInfo::PluginsPath);
-
-        QStringList actual = QApplication::libraryPaths();
-        actual.sort();
-
-        QStringList expected;
-        expected << installPathPlugins << appDirPath;
-        expected = QSet<QString>(expected.constBegin(), expected.constEnd()).values();
-        expected.sort();
-
-        QVERIFY2(isPathListIncluded(actual, expected),
-                 qPrintable("actual:\n - " + actual.join("\n - ") +
-                            "\nexpected:\n - " + expected.join("\n - ")));
-
-        // setting the library paths overrides everything
-         QApplication::setLibraryPaths(QStringList() << testDir);
-        QVERIFY2(isPathListIncluded(QApplication::libraryPaths(), (QStringList() << testDir)),
-                 qPrintable("actual:\n - " + QApplication::libraryPaths().join("\n - ") +
-                            "\nexpected:\n - " + testDir));
-    }
-    {
-        qCDebug(lcTests) << "Initial library path:" << QApplication::libraryPaths();
-
-        int count = QApplication::libraryPaths().size();
-#if 0
-        // this test doesn't work if KDE 4 is installed
-        QCOMPARE(count, 1); // before creating QApplication, only the PluginsPath is in the libraryPaths()
-#endif
-        QString installPathPlugins =  QLibraryInfo::path(QLibraryInfo::PluginsPath);
-        QApplication::addLibraryPath(installPathPlugins);
-        qCDebug(lcTests) << "installPathPlugins" << installPathPlugins;
-        qCDebug(lcTests) << "After adding plugins path:" << QApplication::libraryPaths();
-        QCOMPARE(QApplication::libraryPaths().size(), count);
-        QApplication::addLibraryPath(testDir);
-        QCOMPARE(QApplication::libraryPaths().size(), count + 1);
-
-        // creating QApplication adds the applicationDirPath to the libraryPath
-        int argc = 1;
-        QApplication app(argc, &argv0);
-        QString appDirPath = QCoreApplication::applicationDirPath();
-        qCDebug(lcTests) << QApplication::libraryPaths();
-        // On Windows CE these are identical and might also be the case for other
-        // systems too
-        if (appDirPath != installPathPlugins)
-            QCOMPARE(QApplication::libraryPaths().size(), count + 2);
-    }
-    {
-        int argc = 1;
-        QApplication app(argc, &argv0);
-
-        qCDebug(lcTests) << "Initial library path:" << QCoreApplication::libraryPaths();
-        int count = QCoreApplication::libraryPaths().size();
-        QString installPathPlugins =  QLibraryInfo::path(QLibraryInfo::PluginsPath);
-        QCoreApplication::addLibraryPath(installPathPlugins);
-        qCDebug(lcTests) << "installPathPlugins" << installPathPlugins;
-        qCDebug(lcTests) << "After adding plugins path:" << QCoreApplication::libraryPaths();
-        QCOMPARE(QCoreApplication::libraryPaths().size(), count);
-
-        QString appDirPath = QCoreApplication::applicationDirPath();
-
-        QCoreApplication::addLibraryPath(appDirPath);
-        QCoreApplication::addLibraryPath(appDirPath + "/..");
-        qCDebug(lcTests) << "appDirPath" << appDirPath;
-        qCDebug(lcTests) << "After adding appDirPath && appDirPath + /..:" << QCoreApplication::libraryPaths();
-        QCOMPARE(QCoreApplication::libraryPaths().size(), count + 1);
-#ifdef Q_OS_MACOS
-        QCoreApplication::addLibraryPath(appDirPath + "/../MacOS");
-#else
-        QCoreApplication::addLibraryPath(appDirPath + "/tmp/..");
-#endif
-        qCDebug(lcTests) << "After adding appDirPath + /tmp/..:" << QCoreApplication::libraryPaths();
-        QCOMPARE(QCoreApplication::libraryPaths().size(), count + 1);
-    }
-}
-
-void tst_QApplication::libraryPaths_qt_plugin_path()
-{
-    int argc = 1;
-
-    QApplication app(argc, &argv0);
-    QString appDirPath = QCoreApplication::applicationDirPath();
-
-    // Our hook into libraryPaths() initialization: Set the QT_PLUGIN_PATH environment variable
-    QString installPathPluginsDeCanon = appDirPath + QString::fromLatin1("/tmp/..");
-    QByteArray ascii = QFile::encodeName(installPathPluginsDeCanon);
-    qputenv("QT_PLUGIN_PATH", ascii);
-
-    QVERIFY(!QCoreApplication::libraryPaths().contains(appDirPath + QString::fromLatin1("/tmp/..")));
-}
-
-void tst_QApplication::libraryPaths_qt_plugin_path_2()
-{
-#ifdef Q_OS_UNIX
-    QByteArray validPath = QDir("/tmp").canonicalPath().toLatin1();
-    QByteArray nonExistentPath = "/nonexistent";
-    QByteArray pluginPath = validPath + ':' + nonExistentPath;
-#elif defined(Q_OS_WIN)
-    QByteArray validPath = "C:\\windows";
-    QByteArray nonExistentPath = "Z:\\nonexistent";
-    QByteArray pluginPath = validPath + ';' + nonExistentPath;
-#endif
-
-    {
-        // Our hook into libraryPaths() initialization: Set the QT_PLUGIN_PATH environment variable
-        qputenv("QT_PLUGIN_PATH", pluginPath);
-
-        int argc = 1;
-
-        QApplication app(argc, &argv0);
-
-        // library path list should contain the default plus the one valid path
-        QStringList expected =
-            QStringList()
-            << QLibraryInfo::path(QLibraryInfo::PluginsPath)
-            << QDir(QCoreApplication::applicationDirPath()).canonicalPath()
-            << QDir(QDir::fromNativeSeparators(QString::fromLatin1(validPath))).canonicalPath();
-
-        QVERIFY2(isPathListIncluded(QCoreApplication::libraryPaths(), expected),
-                 qPrintable("actual:\n - " + QCoreApplication::libraryPaths().join("\n - ") +
-                            "\nexpected:\n - " + expected.join("\n - ")));
-    }
-
-    {
-        int argc = 1;
-
-        QApplication app(argc, &argv0);
-
-        // library paths are initialized by the QApplication, setting
-        // the environment variable here doesn't work
-        qputenv("QT_PLUGIN_PATH", pluginPath);
-
-        // library path list should contain the default
-        QStringList expected =
-            QStringList()
-            << QLibraryInfo::path(QLibraryInfo::PluginsPath)
-            << QCoreApplication::applicationDirPath();
-        QVERIFY(isPathListIncluded(QCoreApplication::libraryPaths(), expected));
-
-        qputenv("QT_PLUGIN_PATH", nullptr);
-    }
-}
-#endif
-
 #ifdef QT_BUILD_INTERNAL
 class SendPostedEventsTester : public QObject
 {
@@ -1191,6 +994,32 @@ void tst_QApplication::sendPostedEvents()
 }
 #endif
 
+#if QT_CONFIG(process)
+#if defined(Q_OS_WIN)
+#  define EXE ".exe"
+#else
+#  define EXE ""
+#endif
+void tst_QApplication::runHelperTest()
+{
+#  ifdef Q_OS_ANDROID
+    QSKIP("Skipped on Android: helper not present");
+#  endif
+    int argc = 0;
+    QCoreApplication app(argc, nullptr);
+    QProcess process;
+    process.start(QFINDTESTDATA("apphelper" EXE), { QTest::currentTestFunction() });
+    QVERIFY2(process.waitForFinished(5000), qPrintable(process.errorString()));
+    if (qint8(process.exitCode()) == -1)
+        QSKIP("Process requested skip: " + process.readAllStandardOutput().trimmed());
+
+    QCOMPARE(process.readAllStandardError(), QString());
+    QCOMPARE(process.exitStatus(), QProcess::NormalExit);
+    QCOMPARE(process.exitCode(), 0);
+}
+#undef EXE
+#endif
+
 void tst_QApplication::thread()
 {
     QThread *currentThread = QThread::currentThread();
@@ -2641,7 +2470,7 @@ void tst_QApplication::qtbug_12673()
     QVERIFY(testProcess.waitForFinished(20000));
     QCOMPARE(testProcess.exitStatus(), QProcess::NormalExit);
 #else
-    QSKIP( "No QProcess support", SkipAll);
+    QSKIP("No QProcess support");
 #endif
 }
 
